"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.simplifyCommonFieldsPlugin = exports.expandCommonFieldsPlugin = void 0;
const albumDataFields = [
    'album',
    'albumOrder',
    'albumArtists',
    'genres',
    'year',
    'coverImage',
];
/** 将第一个 metadata 的公共字段复制给后续的 metadata */
const expandCommonFieldsPlugin = () => {
    let firstMetadata;
    return ({ metadata, index }) => {
        if (index === 0) {
            firstMetadata = metadata;
        }
        if (index > 0) {
            albumDataFields.forEach(field => {
                if (!metadata[field]) {
                    metadata[field] = firstMetadata[field];
                }
            });
        }
    };
};
exports.expandCommonFieldsPlugin = expandCommonFieldsPlugin;
const simplifyCommonFieldsPlugin = () => {
    let firstMetadata;
    const isMetadataFieldEqual = (value, compareWith) => {
        if (Array.isArray(value) && Array.isArray(compareWith)) {
            return value.every((item, index) => compareWith[index] === item);
        }
        if (typeof value === 'string' && typeof compareWith === 'string') {
            return value === compareWith;
        }
        return false;
    };
    return ({ metadata, index }) => {
        if (index === 0) {
            firstMetadata = metadata;
        }
        if (index > 0) {
            albumDataFields.forEach(field => {
                if (metadata[field] && isMetadataFieldEqual(metadata[field], firstMetadata[field])) {
                    delete metadata[field];
                }
            });
        }
    };
};
exports.simplifyCommonFieldsPlugin = simplifyCommonFieldsPlugin;
