"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.simplifyMetadataInfoWithArtists = exports.simplifyMetadataInfo = exports.expandMetadataInfoWithoutCover = exports.expandMetadataInfo = void 0;
const alt_names_1 = require("./alt-names");
const common_fields_1 = require("./common-fields");
const cover_1 = require("./cover");
const number_1 = require("./number");
const artists_1 = require("./artists");
const internalNormalize = async (params) => {
    const { plugins, metadatas, cover } = params;
    if (!metadatas || metadatas.length === 0) {
        return metadatas;
    }
    const pluginInstances = plugins.map(p => p({ cover }));
    const results = await Promise.all(metadatas.map(async (metadata, index) => {
        for (const instance of pluginInstances) {
            await instance({
                metadata,
                index,
            });
        }
        return metadata;
    }));
    return results;
};
/** 为简化的 Metadata JSON 填充完整信息 */
const expandMetadataInfo = async (params) => {
    const plugins = [
        cover_1.expandCoverPlugin,
        artists_1.expandArtistsPlugin,
        number_1.expandNumberPlugin,
        common_fields_1.expandCommonFieldsPlugin,
        alt_names_1.altNamesPlugin,
    ];
    return internalNormalize({ plugins, ...params });
};
exports.expandMetadataInfo = expandMetadataInfo;
/** 为简化的 Metadata JSON 填充完整信息, 但不处理封面图 */
const expandMetadataInfoWithoutCover = async (params) => {
    const plugins = [
        artists_1.expandArtistsPlugin,
        number_1.expandNumberPlugin,
        common_fields_1.expandCommonFieldsPlugin,
        alt_names_1.altNamesPlugin,
    ];
    return internalNormalize({ plugins, ...params });
};
exports.expandMetadataInfoWithoutCover = expandMetadataInfoWithoutCover;
/** 对 Metadata JSON 进行简化 */
const simplifyMetadataInfo = async (params) => {
    const plugins = [common_fields_1.simplifyCommonFieldsPlugin, number_1.simplifyNumberPlugin];
    return internalNormalize({ plugins, ...params });
};
exports.simplifyMetadataInfo = simplifyMetadataInfo;
/** 对 Metadata JSON 进行简化, 允许合并相同的 artists 和 composers (用于原创曲) */
const simplifyMetadataInfoWithArtists = async (params) => {
    const plugins = [common_fields_1.simplifyCommonFieldsPlugin, number_1.simplifyNumberPlugin, artists_1.simplifyArtistsPlugin];
    return internalNormalize({ plugins, ...params });
};
exports.simplifyMetadataInfoWithArtists = simplifyMetadataInfoWithArtists;
