import { LyricConfig } from '../../../core-config';
export declare abstract class LyricParser {
    protected table: Element;
    config: LyricConfig;
    protected rows: Element[];
    protected rowData: {
        originalData: Element;
        translatedData: Element;
        hasTranslatedData: boolean;
        time: string;
    }[];
    protected get firstRow(): Element;
    protected get firstRowData(): {
        originalData: Element;
        translatedData: Element;
        hasTranslatedData: boolean;
        time: string;
    };
    constructor(table: Element, config: LyricConfig);
    readLyric(): string;
    protected abstract readLyricRow(row: Element): string;
    protected getRowData(row: Element): {
        originalData: Element;
        translatedData: Element;
        hasTranslatedData: boolean;
        time: string;
    };
    protected readEmptyRow(row: Element): string;
    abstract findLanguage(): string | undefined;
    abstract getLrcFileSuffix(): string;
}
declare class OriginalLyricParser extends LyricParser {
    findLanguage(): string | undefined;
    readLyricRow(row: Element): string;
    getLrcFileSuffix(): string;
}
declare class TranslatedLyricParser extends LyricParser {
    findLanguage(): string | undefined;
    readLyricRow(row: Element): string;
    getLrcFileSuffix(): string;
}
declare class MixedLyricParser extends LyricParser {
    findLanguage(): string | undefined;
    readLyricRow(row: Element): string;
    getLrcFileSuffix(): string;
}
export declare const getLyricParser: (table: Element, config: LyricConfig) => OriginalLyricParser | TranslatedLyricParser | MixedLyricParser;
export {};
