"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.thbWikiCache = exports.thbWiki = exports.ThbWiki = void 0;
const axios_1 = __importDefault(require("axios"));
const jsdom_1 = require("jsdom");
const metadata_source_1 = require("../metadata-source");
const debug_1 = require("../../debug");
const alt_names_1 = require("../alt-names");
class ThbWiki extends metadata_source_1.MetadataSource {
    host;
    constructor(host = 'thwiki.cc') {
        super();
        this.host = host;
    }
    cache = new Map();
    // /api.php?action=opensearch&format=json&search=kappa&limit=20&suggest=true
    async resolveAlbumName(albumName) {
        const url = `https://${this.host}/api.php?action=opensearch&format=json&formatversion=2&search=${encodeURIComponent(albumName)}&limit=20&suggest=true`;
        const response = await axios_1.default.get(url, {
            responseType: 'json',
            timeout: this.config.timeout * 1000,
        });
        if (response.status === 200 && Array.isArray(response.data) && response.data.length > 1) {
            const [, names] = response.data;
            const [name] = names.filter(it => !it.startsWith('歌词:'));
            if (name === albumName) {
                return name;
            }
            return names;
        }
        return [];
    }
    async getAlbumCover(img) {
        const src = img.src.replace('/thumb/', '/');
        const url = src.substring(0, src.lastIndexOf('/'));
        const response = await axios_1.default.get(url, {
            responseType: 'arraybuffer',
            timeout: this.config.timeout * 1000,
        });
        return response.data;
    }
    getAlbumData(infoTable) {
        function getTableItem(labelName, multiple = false) {
            const labelElements = [...infoTable.querySelectorAll('.label')].filter(it => it.innerHTML.trim() === labelName);
            if (labelElements.length === 0) {
                return '';
            }
            const [item] = labelElements.map(it => {
                const nextElement = it.nextElementSibling;
                if (multiple) {
                    return [...nextElement.querySelectorAll('a')].map(element => element.textContent);
                }
                return nextElement.textContent.trim();
            });
            return item;
        }
        const album = getTableItem('名称');
        const albumOrder = getTableItem('编号');
        const albumArtists = getTableItem('制作方', true);
        const genres = getTableItem('风格类型').split('，');
        const year = parseInt(getTableItem('首发日期')).toString();
        return {
            album,
            albumOrder,
            albumArtists,
            genres,
            year,
        };
    }
    getRelatedRows(trackNumberRow) {
        const nextElement = trackNumberRow.nextElementSibling;
        if (nextElement === null || nextElement.querySelector('.left') === null) {
            return [];
        }
        return [nextElement, ...this.getRelatedRows(nextElement)];
    }
    parseRelatedRowInfo(trackInfoRow) {
        const defaultInfoParser = (name) => {
            return (data) => {
                const children = [...data.children];
                const brIndex = children.findIndex(it => it.tagName.toLowerCase() === 'br');
                if (brIndex !== -1) {
                    children.slice(brIndex).forEach(e => e.remove());
                }
                let { textContent } = data;
                /*
                  要是这个值就是一个_, THBWiki 会转成一个警告...
                  例如疯帽子茶会'千年战争'中出现的编曲者就有一个_ (现已更名为'底线')
                  https://thwiki.cc/%E5%8D%83%E5%B9%B4%E6%88%98%E4%BA%89%EF%BD%9Eiek_loin_staim_haf_il_dis_o-del_al
                */
                const warningMatch = textContent.match(/包含无效字符或不完整，并因此在查询或注释过程期间导致意外结果。\[\[(.+)\]\]/);
                if (warningMatch) {
                    textContent = warningMatch[1];
                }
                return {
                    name,
                    result: textContent.trim().split('，'),
                };
            };
        };
        const label = trackInfoRow.querySelector('.label').textContent.trim();
        const rawData = trackInfoRow.querySelector('.text');
        const actions = {
            编曲: defaultInfoParser('arrangers'),
            再编曲: defaultInfoParser('remix'),
            作曲: defaultInfoParser('composers'),
            剧本: defaultInfoParser('scripts'),
            演唱: defaultInfoParser('vocals'),
            翻唱: defaultInfoParser('coverVocals'),
            和声: defaultInfoParser('harmonyVocals'),
            伴唱: defaultInfoParser('accompanyVocals'),
            合唱: defaultInfoParser('chorusVocals'),
            // 演奏: defaultInfoParser('instruments'),
            作词: defaultInfoParser('lyricists'),
            配音: data => {
                const name = 'voices';
                const rows = data.innerHTML.split('<br>').map(it => {
                    const { document } = new jsdom_1.JSDOM(it).window;
                    const anchors = [...document.querySelectorAll('a:not(.external)')];
                    const artists = anchors.map(a => {
                        const isRealArtist = a.previousSibling &&
                            a.previousSibling.textContent === '（' &&
                            a.nextSibling &&
                            a.nextSibling.textContent === '）';
                        if (isRealArtist) {
                            return a.textContent;
                        }
                        return '';
                    });
                    if (artists.every(a => a === '')) {
                        return anchors.map(a => a.textContent);
                    }
                    return artists.filter(a => a !== '');
                });
                // log(rows.flat())
                return {
                    name,
                    result: rows.flat(),
                };
            },
            演奏: data => {
                const name = 'instruments';
                const rows = data.innerHTML
                    .split('<br>')
                    .map(it => {
                    const [instrument, performer] = it
                        .trim()
                        .split('：')
                        .map(row => {
                        return new jsdom_1.JSDOM(row).window.document.body.textContent;
                    });
                    return performer || instrument;
                })
                    .flatMap(row => row.split('，'));
                return {
                    name,
                    result: rows,
                };
            },
            原曲: data => {
                let result = `原曲: `;
                const sources = [...data.querySelectorAll('.ogmusic,.source')];
                sources.forEach((element, index) => {
                    const comma = ', ';
                    if (element.classList.contains('ogmusic')) {
                        result += element.textContent.trim();
                        // 后面还有原曲时加逗号
                        if (index < sources.length - 1 && sources[index + 1].classList.contains('ogmusic')) {
                            result += comma;
                        }
                    }
                    else {
                        // .source
                        result += ` (${element.textContent.trim()})`;
                        // 不是最后一个source时加逗号
                        if (index !== sources.length - 1) {
                            result += comma;
                        }
                    }
                });
                return {
                    name: 'comments',
                    result,
                };
            },
        };
        const action = actions[label];
        if (!action) {
            return { name: 'other', result: '' };
        }
        return action(rawData);
    }
    rowDataNormalize(rowData, removePatterns = []) {
        const normalizeAction = (str) => {
            if (alt_names_1.altNames.has(str)) {
                return alt_names_1.altNames.get(str);
            }
            return (removePatterns
                .reduce((previous, current) => previous.replace(current, ''), str)
                .replace(/\u200b/g, '') // zero-width space
                // eslint-disable-next-line no-irregular-whitespace
                .replace(/　/g, ' ')
                .replace(/([^\s])([(])/g, '$1 $2')
                .replace(/([)])([^\s])/g, '$1 $2')
                .replace(/([^\s]) ([（])/g, '$1$2')
                .replace(/([）]) ([^\s])/g, '$1$2')
                .replace(/’/g, "'")
                .trim());
        };
        for (const [key, value] of Object.entries(rowData)) {
            if (typeof value === 'string') {
                rowData[key] = normalizeAction(value);
            }
            if (Array.isArray(value)) {
                rowData[key] = value.map(v => normalizeAction(v));
            }
        }
        return rowData;
    }
    async parseRow(trackNumberElement) {
        const trackNumber = parseInt(trackNumberElement.textContent).toString();
        const trackNumberRow = trackNumberElement.parentElement;
        const title = trackNumberRow.querySelector('.title').textContent.trim();
        const { lyricLanguage, lyric } = await (async () => {
            const lyricLink = trackNumberRow.querySelector(':not(.new) > a:not(.external)');
            if (this.config.lyric && lyricLink) {
                const { downloadLyrics } = await Promise.resolve().then(() => __importStar(require('./lyrics/thb-wiki-lyrics')));
                return downloadLyrics(`https://${this.host}${lyricLink.href}`, title, this.config);
            }
            return {
                lyric: undefined,
                lyricLanguage: undefined,
            };
        })();
        const relatedInfoRows = this.getRelatedRows(trackNumberRow);
        const infos = relatedInfoRows.map(it => this.parseRelatedRowInfo(it));
        const [lyricists] = infos.filter(it => it.name === 'lyricists').map(it => it.result);
        const [comments] = infos.filter(it => it.name === 'comments').map(it => it.result);
        const arrangers = ['remix', 'arrangers', 'scripts'].flatMap(name => infos
            .filter(it => it.name === name)
            .map(it => it.result)
            .flat());
        const performers = [
            'vocals',
            'coverVocals',
            'harmonyVocals',
            'accompanyVocals',
            'chorusVocals',
            'instruments',
            'voices',
        ].flatMap(name => infos
            .filter(it => it.name === name)
            .map(it => it.result)
            .flat());
        const [composers] = infos.filter(it => it.name === 'composers').map(it => it.result);
        // log('artists:', artists)
        if (arrangers.length === 0 && composers) {
            arrangers.push(...composers);
        }
        const artists = [...new Set(performers.concat(arrangers))];
        const artistsRowData = {
            artists,
            lyricists,
            composers,
        };
        const otherRowData = {
            title,
            trackNumber,
            comments,
            lyric,
            lyricLanguage,
        };
        const rowData = {
            ...this.rowDataNormalize(artistsRowData, [/（.+）$/]),
            ...this.rowDataNormalize(otherRowData),
        };
        (0, debug_1.log)(rowData);
        return rowData;
    }
    async getMetadata(albumName, cover) {
        // const url = `https://${this.host}/index.php?search=${encodeURIComponent(albumName)}`
        const url = `https://${this.host}/${encodeURIComponent(albumName)}`;
        const response = await axios_1.default.get(url, { timeout: this.config.timeout * 1000 });
        const dom = new jsdom_1.JSDOM(response.data);
        const { document } = dom.window;
        const infoTable = document.querySelector('.doujininfo');
        if (!infoTable) {
            throw new Error('页面不是同人专辑词条');
        }
        const { album, albumOrder, albumArtists, genres, year } = this.getAlbumData(infoTable);
        const coverImageElement = document.querySelector('.cover-artwork img');
        const coverImage = await (async () => {
            if (cover) {
                return cover;
            }
            if (coverImageElement) {
                return this.getAlbumCover(coverImageElement);
            }
            return undefined;
        })();
        const musicTables = [...document.querySelectorAll('.musicTable')];
        let discNumber = 1;
        const metadatas = [];
        // const getAlbumOrder = (disc: number) => {
        //   const splitAlbumOrders = albumOrder.split(/\s\+\s/)
        //   if (disc > splitAlbumOrders.length) {
        //     return splitAlbumOrders[splitAlbumOrders.length - 1]
        //   }
        //   return splitAlbumOrders[disc - 1]
        // }
        for (const table of musicTables) {
            const trackNumbers = [
                ...table.querySelectorAll('tr > td[class^="info"]'),
            ];
            for (const trackNumberElement of trackNumbers) {
                const metadata = {
                    discNumber: discNumber.toString(),
                    album,
                    albumOrder,
                    // albumOrder: getAlbumOrder(discNumber),
                    albumArtists,
                    genres,
                    year,
                    coverImage,
                    ...(await this.parseRow(trackNumberElement)),
                };
                metadatas.push(metadata);
            }
            discNumber += 1;
        }
        return metadatas;
    }
}
exports.ThbWiki = ThbWiki;
exports.thbWiki = new ThbWiki();
exports.thbWikiCache = new ThbWiki('cache.thwiki.cc');
