"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.flacReader = exports.FlacReader = void 0;
const flac_tagger_1 = require("flac-tagger");
const metadata_reader_1 = require("../metadata-reader");
class FlacReader extends metadata_reader_1.MetadataReader {
    async readRaw(input) {
        return (0, flac_tagger_1.readFlacTags)(input);
    }
    async read(input) {
        const tag = typeof input === 'string' || input instanceof Buffer ? await this.readRaw(input) : input;
        const { tagMap, picture } = tag;
        const readVorbisComments = ((name, asList) => {
            const comments = tagMap[name];
            if (asList) {
                if (Array.isArray(comments)) {
                    return comments.length === 0 ? undefined : comments;
                }
                return comments ? [comments] : undefined;
            }
            if (Array.isArray(comments)) {
                return comments[0];
            }
            return comments || undefined;
        });
        const metadata = {
            title: readVorbisComments('title'),
            artists: readVorbisComments('artist', true),
            album: readVorbisComments('album'),
            albumOrder: readVorbisComments('albumSort'),
            discNumber: readVorbisComments('discNumber'),
            trackNumber: readVorbisComments('trackNumber'),
            composers: readVorbisComments('composer', true),
            genres: readVorbisComments('genre', true),
            year: readVorbisComments('date'),
            lyricists: readVorbisComments('lyricist', true),
            albumArtists: readVorbisComments('albumArtist', true),
            comments: readVorbisComments('comment'),
            lyric: readVorbisComments('lyrics'),
            coverImage: picture.buffer,
        };
        return metadata;
    }
}
exports.FlacReader = FlacReader;
exports.flacReader = new FlacReader();
