"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mp3Writer = exports.Mp3Writer = void 0;
const node_id3_1 = __importDefault(require("node-id3"));
const metadata_writer_1 = require("../metadata-writer");
const debug_1 = require("../../debug");
const languageCodeConvert = (code) => {
    const mapping = {
        ja: 'jpn',
        de: 'deu',
        zh: 'zho',
    };
    return code ? mapping[code] || mapping.ja : mapping.ja;
};
class Mp3Writer extends metadata_writer_1.MetadataWriter {
    getNodeId3Tag(metadata, separator) {
        const tag = {
            title: metadata.title,
            artist: metadata.artists.join(separator),
            album: metadata.album,
            albumOrder: metadata.albumOrder,
            partOfSet: metadata.discNumber,
            trackNumber: metadata.trackNumber,
            composer: metadata.composers ? metadata.composers.join(separator) : '',
            genre: metadata.genres ? metadata.genres.join(separator) : '',
            year: metadata.year || '',
            textWriter: metadata.lyricists ? metadata.lyricists.join(separator) : '',
            performerInfo: metadata.albumArtists ? metadata.albumArtists.join(separator) : '',
            comment: {
                language: this.config.commentLanguage,
                text: metadata.comments || '',
            },
            unsynchronisedLyrics: {
                language: languageCodeConvert(metadata.lyricLanguage),
                text: metadata.lyric || '',
            },
        };
        if (metadata.coverImage) {
            tag.image = {
                type: {
                    id: 3,
                    name: 'front cover',
                },
                description: '',
                imageBuffer: metadata.coverImage,
            };
        }
        return tag;
    }
    async write(metadata, filePath) {
        const tag = this.getNodeId3Tag(metadata, this.config.separator);
        if (this.config.lyric && this.config.lyric.output === 'lrc') {
            tag.unsynchronisedLyrics.text = '';
            tag.unsynchronisedLyrics.language = undefined;
        }
        (0, debug_1.log)(this.config.coverCompressSize, this.config.coverCompressSize * 1024 * 1024, tag.image?.imageBuffer?.length);
        if (tag.image?.imageBuffer) {
            const { compressImageByConfig } = await Promise.resolve().then(() => __importStar(require('../image-compress')));
            tag.image.imageBuffer = await compressImageByConfig(tag.image.imageBuffer, this.config);
        }
        const result = node_id3_1.default.write(tag, filePath);
        if (result !== true) {
            throw new Error(`Write operation failed. filePath = ${filePath}`);
        }
    }
}
exports.Mp3Writer = Mp3Writer;
exports.mp3Writer = new Mp3Writer();
